"""C H A T B O T"""

from .addressbook import main as ab_main
from .notes import main as nb_main
from .sort import main as sr_main
from prettytable import PrettyTable


def main():
    """Main function"""

    while True:
        x = PrettyTable()
        x.field_names = ['C H A T B O T']
        x.align = 'l'
        x.add_row(['0: Exit'])
        x.add_row(['1: AddressBook'])
        x.add_row(['2: NoteBook'])
        x.add_row(['3: Sort files'])
        print(x)
        user_input = input(f"\nEnter command {chr(10151)*3} ")
        if user_input == "1":
            ab_main()
        elif user_input == "2":
            nb_main()
        elif user_input == "3":
            sr_main()
        elif user_input == "0":
            print("\nGood bay!")
            break
        else:
            print(f"I don't understand you!")


if __name__ == "__main__":
    main()



"""S O R T   F I L E"""


import os
import shutil
import re
import rarfile
from .information import start_info_sf

extensions_img = ['.jpeg', '.png', '.jpg', '.svg']
extensions_doc = ['.doc', '.docx', '.txt', '.pdf', '.xlsx', '.pptx']
extensions_video = ['.avi', '.mp4', '.mov', '.mkv']
extensions_music = ['.mp3', '.ogg', '.wav', '.amr']
extensions_archives = ['.zip', '.gz', '.tar', '.rar']
folder_name = ['image', 'text', 'video', 'music', 'archives', 'other']


def normalize(name):
    """Transliteration of files from Cyrillic to Latin"""

    cyrillic_symbols = "абвгдеёжзийклмнопрстуфхцчшщъыьэюяєіїґ"
    translation = (
        "a", "b", "v", "g", "d", "e", "e", "j", "z", "y", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u",
        "f", "h", "ts", "ch", "sh", "sch", "", "y", "", "e", "yu", "ya", "je", "i", "ji", "g")
    trans = {}

    for c, l in zip(cyrillic_symbols, translation):
        trans[ord(c)] = l
        trans[ord(c.upper())] = l.upper()
        
    filename, file_extension = os.path.splitext(name)
    return re.sub(r'\W', '_',
                  filename.translate(trans)) + file_extension


def sort_dir(root_path, current_path, level=0):
    """Recursion through directories"""

    names = os.listdir(current_path)
    for file in names:

        if level == 0 and file in folder_name:
            continue

        if os.path.isdir(current_path+file):
            sort_dir(root_path, current_path+file+'/', level+1)
            shutil.rmtree(current_path+file)
            continue

        if sort_extensions(extensions_img, file, root_path + 'image/', current_path):
            continue
        if sort_extensions(extensions_doc, file, root_path + 'text/', current_path):
            continue
        if sort_extensions(extensions_video, file, root_path + 'video/', current_path):
            continue
        if sort_extensions(extensions_music, file, root_path + 'music/', current_path):
            continue
        if sort_archive(extensions_archives, file, root_path + 'archives/', current_path):
            continue

        if not os.path.exists(root_path + 'other/' + file):
            if not os.path.exists(root_path + 'other/'):
                os.makedirs(root_path + 'other/')
            shutil.move(current_path + file, root_path + 'other/' + file)


def sort_extensions(extensions, file, destination_dir, location_dir):
    """Sorting files by extension"""

    for extension in extensions:
        if extension in file.lower():
            if not os.path.exists(destination_dir):
                os.makedirs(destination_dir)
            shutil.move(location_dir + file,
                        destination_dir + normalize(file))
            return True


def sort_archive(extensions, file, destination_dir, location_dir):
    """Unzipping and sorting archives"""

    for extension in extensions:
        if extension in file.lower():
            if not os.path.exists(destination_dir):
                os.makedirs(destination_dir)
            try:
                if extension == '.rar':
                    r = rarfile.RarFile(location_dir + file)
                    r.extractall(destination_dir +
                                 file.removesuffix(extension))
                    r.close()
                else:
                    shutil.unpack_archive(
                        location_dir + file, destination_dir + file.removesuffix(extension))
                os.remove(location_dir + file)
            except:
                shutil.move(location_dir + file,
                            destination_dir + normalize(file))
            return True


def count_files(dir_path):
    """File counter"""

    count = 0
    for root_dir, cur_dir, files in os.walk(dir_path):
        if files != ['.DS_Store']:
            print(files)
            count += len(files)

    return f'In folder: {count} files'


def main():
    """Main function, which makes a request to enter the path to the directory for sorting"""

    print(start_info_sf())
    while True:
        try:
            root_path = input(f'\nEnter the path to the folder to sort {chr(10151)*3} ')
            if root_path == "0":
                break
            if root_path[-1] != '/':
                root_path += '/'
                sort_dir(root_path, root_path)
                print(f'Everything is sorted  {chr(9989)}')
                print(count_files(root_path))
            else:
                print(f"{chr(128679)} error, try again")

        except:
            print(f'{chr(128679)} error')


if __name__ == "__main__":
    main()



"""N O T E B O O K"""


from collections import UserDict, UserString
from .information import start_info_nb, help_info_nb
from .prompt_tool import Completer, RainbowLexer
from prompt_toolkit import prompt
import pickle


filename = "notebook.bin"


class Tag(UserString):
    pass


class Body(UserString):
    pass


class Title(UserString):
    pass


class Note:

    def __init__(self, title: Title, tag: Tag = None, body: Body = '-'):
        if len(title) > 0 and title[0].isalnum():
            self.title = title
        self.body = body
        self.tags = []
        if tag:
            self.tags.append(tag)

    def __repr__(self):
        return f'Title: {self.title}, Tags: {self.tags}, Body: {self.body}'

    def add_tag(self, tag: Tag):
        """Added tag in note"""

        if tag not in self.tags:
            self.tags.append(tag)
            print(f'tag {tag} added in note {self.note}')
        return self.tags


class NoteBook(UserDict):

    def add_note(self, note: Note):
        """Added note in notebook"""

        if note.title not in self.data:
            self.data[note.title] = note
            return f"Note '{note.title}' was created. {note}"
        else:
            return f'{note.title} already exist'

    def del_note(self, note: Note):
        """Deletes note from notebook"""

        if note.title in self.data:
            self.data.pop(note.title)
            return f'Note {note.title} successfully deleted'
        else:
            return f'note {note.title} is not in NoteBook'

    def change_note(self, note_old: Note, note_new: Note):
        """Replacing on note_old with a note_new in a notebook"""

        if note_old.title in self.data:
            self.data.pop(note_old.title)
            self.data[note_new.title] = note_new
            return f"Note {note_old.title} was change -> new note: {note_new}"
        else:
            return f"Note '{note_old.title}' is not in NoteBook"

    def find(self, find_str: str):
        """Find notes that contain the specified text"""

        str_ret = '\nBy the search key ' + '"' + \
            find_str + '":' + '\n' + '_' * 40 + '\n'
        for value in self.data.values():
            if find_str.lower() in str(value).lower():
                str_ret += str(value) + '\n'
        return str_ret

    def find_sort_tags(self, tags):
        """Search and sort notes by specified tags"""

        dict_ret = {}
        for tag in tags:
            dict_ret[tag] = []
            for value in self.data.values():
                if tag.lower() in value.tags:
                    dict_ret[tag].append(value)
        return dict_ret

    def save_to_file(self, filename):
        """Save notes to the file"""

        with open(filename, "wb") as fh:
            pickle.dump(self, fh)
            print("Notes saved in file")

    def note_iterator(self, n=2):
        """Pagination (page-by-page output) for NoteBook"""

        k = 1
        block = str(k) + '-' * 40
        string_counter = 0
        for note in self.data.values():
            string_counter += 1
            block += '\n' + str(note)
            if string_counter == n:
                k += 1
                block += '\n' + str(k) + '-' * 40
                yield block
                string_counter = 0
                block = ''
        yield block


def decor_error(func):
    """Decorator for handling exceptions """

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except IndexError:
            return "IndexError... Enter the correct numbers of attributes, please"
        except KeyError:
            return "KeyError..."
        except ValueError:
            return "ValueError..."
        except AttributeError:
            return "AttributeError... Enter the correct attribute, please"
    return wrapper


def read_from_file(filename) -> NoteBook:
    """Uploading data from a file"""

    try:
        with open(filename, "rb") as fh:
            result = pickle.load(fh)
            print('Read NoteBook from file')
            return result
    except FileNotFoundError:
        print(f'{filename} not exist')
        return NoteBook()


def hello(*args, **kwargs: NoteBook):
    return f"{chr(129299)} How can I help you?\n"


@decor_error
def add_note(*args, **kwargs: NoteBook):
    """Added note in notebook"""

    nb = kwargs.get('nb')
    title = Title(args[0])
    tag = None
    body = '-'
    if len(args) > 1:
        tag = Tag(args[1])
    if len(args) > 2:
        body = ' '.join(args[2:])
    note = Note(title, tag, body)
    return nb.add_note(note)


@decor_error
def del_note(*args, **kwargs: NoteBook):
    """Deletes note from notebook"""

    nb = kwargs.get('nb')
    note = Note(args[0])
    return nb.del_note(note)


@decor_error
def change_note(*args, **kwargs: NoteBook):
    """Replacing on note_old with a note_new in a notebook"""

    nb = kwargs.get('nb')
    note_old = Note(args[0])
    note_new = Note(args[1])
    return nb.change_note(note_old, note_new)


@decor_error
def add_tag(*args, **kwargs: NoteBook):
    """Added tag in note"""

    nb = kwargs.get('nb')
    note = nb[args[0]]
    tag = args[1]
    print(f'tag {tag} added in note {note}')
    return note.add_tag(tag)


@decor_error
def find(*args, **kwargs: NoteBook):
    """Find notes that contain the specified text"""

    nb = kwargs.get('nb')
    find_str = args[0]
    return nb.find(find_str)


@decor_error
def find_sort_tags(*args, **kwargs: NoteBook):
    """Search and sort notes by specified tags"""

    nb = kwargs.get('nb')
    tags = list(args)
    return nb.find_sort_tags(tags)


@decor_error
def show_all(*args, **kwargs: NoteBook):
    """Display the contents of a NoteBook"""

    nb = kwargs.get('nb')
    result = f'Notes list:\n'
    iter = nb.note_iterator()
    for item in iter:
        result += item
    return result


def exit_save_change(nb: NoteBook):
    """Request to save information"""

    while True:
        user_input_save = input("Save change? y/n: ")
        if user_input_save == "y":
            nb.save_to_file(filename)
            break
        elif user_input_save == "n":
            break
        else:
            continue
    print("Good bye!")


"""Dictionary with commands(key - function: value - command)"""

COMMANDS = {
    hello: "hello",
    add_note: "add",
    del_note: "del",
    change_note: "change",
    add_tag: "tag+",
    find: "find",
    find_sort_tags: "tags",
    show_all: "show",
    help_info_nb: "info"
}


def parser_command(user_input: str):
    """The function parses the string entered by the user, splits it into a command and other arguments"""

    for command, key_word in COMMANDS.items():
        if user_input.lower().split()[0] == key_word:
            return command, user_input.replace(key_word, "").strip().split()
    return None, None


def main():
    """Main function"""

    print(start_info_nb())
    nb = read_from_file(filename)

    while True:
        user_input = prompt("Enter command>>>", completer=Completer, lexer=RainbowLexer())
        if user_input:
            if user_input.lower() in ["close", "exit", "."]:
                exit_save_change(nb)
                break
            command, data = parser_command(user_input)
            if not command:
                print("Sorry, I don't understand you!\n")
            else:
                print(command(*data, nb=nb))
        else:
            print("Sorry, I don't understand you!\nEnter: 'info'\n")


if __name__ == "__main__":
    main()


"""Information output functions"""

from prettytable import PrettyTable


def help_info_ab(*args, **kwargs):
    """help info for AddressBook"""

    print(f"I N F O:")
    x = PrettyTable()
    x.field_names = ['commands', 'description', 'example']
    x.align = 'l'
    x.add_row(['>hello<', 'hello', '>command<'])
    x.add_row(['>add<', 'add name and phone, if there is a name add phone', '>command< >name< >phone<'])
    x.add_row(['>change<', 'change phone', '>command< >name< >old_phone< >new_phone<'])
    x.add_row(['>show<', 'show all AddressBook', '>command<'])
    x.add_row(['>phone<', 'show phone', '>command< >name<'])
    x.add_row(['>del<', 'del contact', '>command< >name<'])
    x.add_row(['>birth<', 'add birthday', '>command< >name< >date<'])
    x.add_row(['>email<', 'add email', '>command< >name< >email<'])
    x.add_row(['>address<', 'add address', '>command< >name< >address<'])
    x.add_row(['>find<', 'search by matches', '>command< >target<'])
    x.add_row(['>nextbirth<', 'next birthday for >n< number of days', '>command< >next number days<'])
    x.add_row(['>info<', 'information', '>command<'])
    x.add_row(['>., close, exit<', 'exit', '>command<'])
    return x


def help_info_nb(*args, **kwargs):
    """help info for NoteBook"""

    print(f"I N F O:")
    x = PrettyTable()
    x.field_names = ['commands', 'description', 'example']
    x.align = 'l'
    x.add_row(['>hello<', 'hello', '>command<'])
    x.add_row(['>add<', 'add new note', '>command< >title< >tag< >body<'])
    x.add_row(['>del<', 'delete note', '>command< >note<'])
    x.add_row(['>change<', 'change note', '>command< >note_old< >note_new<'])
    x.add_row(['>show<', 'show all NoteBook', '>command<'])
    x.add_row(['>tag+<', 'add title', '>command< >note< >tag<'])
    x.add_row(['>find<', 'find notes', '>command< >title< >target<'])
    x.add_row(['>tags<', 'find and sort by tags', '>command< >tag[, tag,tag,...,tag]<'])
    x.add_row(['>info<', 'information', '>command<'])
    x.add_row(['>., close, exit<', 'exit', '>command<'])
    return x


def start_info_ab():
    """start info for AddressBook"""

    x = PrettyTable()
    x.field_names = [" A D D R E S S B O O K "]
    x.align = 'l'
    x.add_row([f"enter: info{chr(128227)}"])
    return x


def start_info_nb():
    """start info for NoteBook"""

    x = PrettyTable()
    x.field_names = [" N O T E B O O K "]
    x.align = 'l'
    x.add_row([f"enter: info{chr(128227)}"])
    return x


def start_info_sf():

    x = PrettyTable()
    x.field_names = [" S O R T I N G   F I L E S "]
    x.align = 'l'
    x.add_row([f"Exit: 0"])
    return x



from prompt_toolkit.lexers import Lexer
from prompt_toolkit.styles.named_colors import NAMED_COLORS
from prompt_toolkit.completion import NestedCompleter


class RainbowLexer(Lexer):
    def lex_document(self, document):
        colors = list(sorted({"Teal": "#008080"}, key=NAMED_COLORS.get))

        def get_line(lineno):
            return [
                (colors[i % len(colors)], c)
                for i, c in enumerate(document.lines[lineno])
            ]

        return get_line


Completer = NestedCompleter.from_nested_dict({'hello': None, 'exit': None, 'close': None,
                                              'change': None, 'phone': None, 'show': None,
                                              'del': None, 'birth': None, 'email': None,
                                              'nextbirth': None, 'find': None, 'info': None,
                                              'add': None, 'address': None,  'tags': None,
                                              'tag+': None, '.': None})



from setuptools import setup, find_namespace_packages


setup(name='Chatbot_by_IE',
      version='0.1.7',
      description='Chatbot: Addressbook, NoteBook, Sort file',
      url='https://github.com/Sanyavas/chatbot-team-project.git',
      author='Oleksandr Vasylyna, Oleh Vakulchyk, Nataleia Orlovska, Anton Sokhnenko, Polina Dyka',
      author_email='vasilinaoleksanrd@gmail.com',
      license='MIT',
      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent"],
      packages=find_namespace_packages(),
      install_requires=["prompt_toolkit", "rarfile", "prettytable"],
      entry_points={"console_scripts": [
            "chatbot=app.menu:main"]}
      )



"""A D D R E S S B O O K"""

from collections import UserDict
from datetime import datetime
from .information import start_info_ab, help_info_ab
import pickle
from prettytable import PrettyTable
from prompt_toolkit import prompt
from .prompt_tool import Completer, RainbowLexer
import re

filename = "addressbook.bin"


class AddressBook(UserDict):
    def add_record(self, record):
        self.data[record.name.value] = record

    def iterator(self, n=1):
        index = 1
        print_block = '-' * 100 + '\n'
        for record in self.data.values():
            print_block += str(record) + '\n'
            if index < n:
                index += 1
            else:
                yield print_block
                index, print_block = 1, '-' * 100 + '\n'
        yield print_block


class Filed:
    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return f"{self.value}"


class Name(Filed):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, new_value):
        self._value = new_value.title()


class Phone(Filed):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, new_value):
        new_value = (
            new_value.strip()
            .removeprefix("+")
            .replace("(", "")
            .replace(")", "")
            .replace("-", "")
            .replace(" ", "")
        )
        if new_value.isdigit():
            if len(new_value) == 12:
                new_value = "+" + new_value
                self._value = new_value
            elif len(new_value) == 10:
                new_value = "+38" + new_value
                self._value = new_value
        else:
            print(f"{chr(128679)} Entered wrong {chr(128222)}phone: {new_value}, correct phone: 0674523698")


class Birthday(Filed):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, birthday):
        try:
            new_value = datetime.strptime(birthday, "%d.%m.%Y").date()
            self._value = new_value
        except ValueError:
            print(f"{chr(128679)} ValueError! Enter correct {chr(128198)}date %dd.%mm.%yyyy")


class Email(Filed):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, email):
        new_email = re.search(r"[a-zA-Z0-9][\w._]+@\w+\.\w{2,}", email)
        if new_email:
            self._value = email
        else:
            print(f"{chr(128679)} You entered wrong {chr(128231)}email! >> {new_email}")


class Address(Filed):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, new_address):
        self._value = new_address.title()


class Record:
    def __init__(self, name, *phones):
        self.name = name
        self.phones = list(phones)
        self.birthday = "-"
        self.email = "-"
        self.address = "-"

    def __repr__(self):
        return f"name: {self.name}, phone: {self.phones}, email: {self.email}," \
               f" address: {self.address}, birthday: {self.birthday}"

    def add_phone(self, phones):
        self.phones.append(phones)

    def change_phone(self, phone_old, phone_new):
        self.phones.remove(phone_old)
        self.phones.append(phone_new)

    def remove_contact(self, phones):
        self.phones.remove(phones)

    def add_birthday(self, birthday):
        self.birthday = birthday

    def add_email(self, email):
        self.email = email

    def add_address(self, address):
        self.address = address

    def days_to_birthday(self, birthday):
        self.birthday = birthday
        date_now = datetime.now().date()
        corr_date = birthday.replace(year=date_now.year)
        if date_now > corr_date:
            corr_date = corr_date.replace(year=date_now.year + 1)
        delta = corr_date - date_now
        return int(delta.days)


def decor_error(func):
    """Decorator for exception handling"""

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except IndexError:
            return f"{chr(128679)} IndexError... Enter name or phone please"
        except KeyError:
            return f"{chr(128679)} KeyError..."
        except ValueError:
            return f"{chr(128679)} ValueError..."
        except AttributeError:
            return f"{chr(128679)} AttributeError..."

    return wrapper


"""The handler functions are responsible for the direct execution of commands"""


@decor_error
def hello(*args, **kwargs: AddressBook):
    return f"{chr(129299)} How can I help you?\n"


@decor_error
def add_phone(*args, **kwargs: AddressBook):
    """Adding a contact and phone, if there is a contact, it adds the phone"""

    ab = kwargs.get('ab')
    name = Name(args[0])
    phone = Phone(args[1])
    rec = ab.get(name.value)
    if rec:
        rec.add_phone(phone.value)
    else:
        rec = Record(name, phone.value)
    ab.add_record(rec)
    return f"{chr(9989)} {name} was created. {chr(128222)}phone: {phone}"


@decor_error
def change(*args, **kwargs: AddressBook):
    """Change phone"""

    ab = kwargs.get('ab')
    name = Name(args[0])
    phone_old = Phone(args[1])
    phone_new = Phone(args[2])
    rec = ab.get(name.value)
    if rec:
        rec.change_phone(phone_old.value, phone_new.value)
        ab.add_record(rec)
        return f"{chr(9989)} {name} was change -> old phone: {phone_old} new phone: {phone_new}"
    else:
        return f"{chr(10062)} Name {name} isn't in the AddressBook"


@decor_error
def phone(*args, **kwargs: AddressBook):
    """Contact phone output"""

    name = Name(args[0])
    ab = kwargs.get('ab')
    rec = ab.get(name.value)
    if rec:
        for value in ab.values():
            return f'{chr(9989)} {name}: {chr(128222)}phone:{value.phones}'
    return f"{chr(10062)} Name {name} isn't in the AddressBook"


@decor_error
def delete(*args, **kwargs: AddressBook):
    """Delete contact"""

    ab = kwargs.get('ab')
    name = Name(args[0])
    rec = ab.get(name.value)
    if rec:
        ab.pop(name.value)
        return f"{chr(9989)} {name} deleted {chr(10060)} "
    return f"{chr(10062)} Name {name} isn't in the AddressBook"


@decor_error
def add_birthday(*args, **kwargs: AddressBook):
    """Adding date of birth"""

    ab = kwargs.get('ab')
    name = Name(args[0])
    bir_day = Birthday(args[1])
    rec = ab.get(name.value)
    if rec:
        rec.add_birthday(bir_day.value)
        ab.add_record(rec)
        return f"{chr(9989)} {name} was add {chr(127874)}birthday: {bir_day}"
    return f"{chr(10062)} Name {name} isn't in the AddressBook"


@decor_error
def add_email(*args, **kwargs: AddressBook):
    """Adding email"""

    ab = kwargs.get('ab')
    name = Name(args[0])
    email = Email(args[1])
    rec = ab.get(name.value)
    if rec:
        rec.add_email(email.value)
        ab.add_record(rec)
        return f"{chr(9989)} {name} was add {chr(128231)}email: {email}"
    return f"{chr(10062)} Name {name} isn't in the AddressBook"


@decor_error
def add_address(*args, **kwargs: AddressBook):
    """Adding address"""

    ab = kwargs.get('ab')
    name = Name(args[0])
    address_row = " ".join(args[1:])
    address = Address(address_row)
    rec = ab.get(name.value)
    if rec:
        rec.add_address(address.value)
        ab.add_record(rec)
        return f"{chr(9989)} {name} was add {chr(127968)}address: {address}"
    return f"{chr(10062)} Name {name} isn't in the AddressBook"


@decor_error
def next_birthdays(*args, **kwargs: AddressBook):
    """Displays a list of contacts whose birthday is between the current date and the specified number of days"""

    ab = kwargs.get('ab')
    days = int(args[0])
    x = PrettyTable()
    for value in ab.values():
        if value.days_to_birthday(value.birthday) <= days:
            years = datetime.now().year - value.birthday.year
            x.field_names = ["name", "birthday", "years", "phone", "email", "address"]
            x.add_row([value.name, value.birthday, years, value.phones, value.email, value.address])
    if x:
        print(x)
    else:
        return f"{chr(10062)} No birthdays for the next {days} days"
    return f"\nTo congratulate on his {chr(127873)}birthday!!!"


@decor_error
def show_all(*args, **kwargs: AddressBook):
    """Displaying the contents of the contact book"""

    x = PrettyTable()
    ab = kwargs.get('ab')
    # result = f'Contacts list:\n'
    # print_list = ab.iterator()
    # for item in print_list:
    #     result += f"{item}"
    print(f"\nContacts list:{chr(128214)}")
    for i in ab.values():
        x.field_names = ["name", "phone", "email", "address", "birthday"]
        x.add_row([i.name, i.phones, i.email, i.address, i.birthday])
    return x


@decor_error
def find(*args, **kwargs: AddressBook):
    """Search the contents of the contact book by several digits of the phone number or letters of the name, etc"""

    ab = kwargs.get('ab')
    s_search = args[0]
    for contact in ab.values():
        contact = str(contact)
        if s_search.lower() in contact.lower():
            print(f'{"-" * 120}\n{contact}')
        if not contact:
            return f"On request <{s_search}> {chr(128373)}don't found contacts"
    return f"{'-' * 120}\nOn request <{s_search}> {chr(128373)}found these contacts"


def exit_save_change(ab: AddressBook):
    """Request to save information"""

    while True:
        user_input_save = input(f"{chr(128221)}Save change? y/n: ")
        if user_input_save == "y":
            save_contacts_to_file(ab)
            break
        elif user_input_save == "n":
            break
        else:
            continue
    print(f"{chr(128075)} Good bye!")


def save_contacts_to_file(contacts):
    """Save to file"""

    with open(filename, 'wb') as file:
        pickle.dump(contacts, file)
        print(f"{chr(9989)} Changes saved.")


def open_contacts_from_file():
    """Loading contacts from a file"""

    try:
        with open(filename, "rb") as file:
            return pickle.load(file)
    except FileNotFoundError:
        return AddressBook()


"""Dictionary with commands (key - function: value - command)"""

COMMANDS = {
    hello: "hello",
    add_phone: "add ",
    change: "change",
    phone: "phone",
    show_all: "show",
    delete: "del",
    add_birthday: "birth",
    add_email: "email",
    add_address: "address",
    next_birthdays: "nextbirth",
    find: "search",
    help_info_ab: "info",
}


def parser_command(user_input: str):
    """The function parses the term entered by the user, divides it into a command and other information"""

    for command, key_word in COMMANDS.items():
        if user_input.lower().startswith(key_word):
            return command, user_input.replace(key_word, "").strip().split(" ")

    return None, None


def main():
    """Main function AddressBook"""
    print(start_info_ab())
    ab = open_contacts_from_file()

    while True:
        user_input = prompt(f"\nEnter command {chr(10151)*3} ", completer=Completer, lexer=RainbowLexer())
        if user_input.lower() in ["close", "exit", "."]:
            exit_save_change(ab)
            break
        command, data = parser_command(user_input)
        if not command:
            print(f"\n{chr(129400)}Sorry, I don't understand you!")
        else:
            print(command(*data, ab=ab))


if __name__ == "__main__":
    main()
